#version 330

//use reflect or refract
#define REFLECT

//do inverse transform
#define INVERSE_TRANSFORM


#define INV_VIEW_NORMAL


layout(std140) uniform Matrices
{
	mat4 m_pvm;
	mat4 m_viewModel;
#ifndef INV_VIEW_NORMAL
	mat4 m_view;
#else
	mat3 m_view;
#endif
	mat3 m_normal;
};

in vec4 position;	// local space
in vec3 normal;		// local space

out vec3 texCoord;

//----------------------------------------------------------------
// TODO:
// 1) Pass position to the fragment shader - renders sphere with image on it.
// 2) Pass reflected view direction in eye space - reflective sphere, camera independent.
// 3) Pass reflected view direction in world space - reflective sphere, camera dependent: texCoord = vec3(inverse(m_view) * vec4(reflectView, 0.f));
// 4) Convert the vector back to the world space by multiplying it with inverse of the view matrix.
// 5) Fix vector flip by detecting and correcting negative determinant.
// 6) Use transpose of normal_view matrix (which is inverse-transpose of the view matrix), use INV_VIEW_NORMAL define in demo.cpp.
// 7) Try refraction - needs closer skybox to be apparent.
// 8) Zoom the sphere - what happens and why?
//----------------------------------------------------------------

void main()
{
#if 0
	//TODO 1)
	// Pass position to the fragment shader
	texCoord = position.xyz;

#else
	// Calculate first in camera / view space.
	vec4 vertexView = m_viewModel * position;
	vec3 normalView = normalize(m_normal * normal);

	vec3 incident = vec3(vertexView);


	#ifdef REFLECT
		//TODO 2)
		//  Compute reflection
		vec3 reflectView = reflect(incident, normalView);
	
	#else
		//TODO 2)
		//  Compute refraction with given ratio of refraction
		vec3 reflectView = refract(incident, normalView, 1.333f);
	#endif

	// Do inverse transform?	
	#ifndef INVERSE_TRANSFORM
	
		//No inverse transform..

		//TODO 2)
		//  Pass reflected view direction in eye space
		//  result will be camera independent
		texCoord = reflectView;
		
	#else
		// inverse transform - go back from view space, as the cube map is in world space.


		#ifdef INV_VIEW_NORMAL


			texCoord = vec3(transpose(m_view) * reflectView);


		#else

			texCoord = vec3(inverse(m_view) * vec4(reflectView, 0.f));

			//TODO 5) Fix vector flip by detecting and correcting negative determinant
			if (determinant(m_view) < 0.f)
				texCoord = -texCoord;

		#endif
	#endif
#endif

	gl_Position = m_pvm * position;
}