#version 330

layout (std140) uniform Materials
{
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	int texCount;
};

layout (std140) uniform Lights
{
	vec3 l_dir;		// camera space
};

in Data
{
	vec3 normal;	// camera space
	vec4 eye;		// camera space
	vec2 texCoord;
} DataIn;

uniform sampler2D texUnit;

uniform float time = 0.5f;

out vec4 outputF;

//----------------------------------------------------------------
// TODO:
// 1) Discard fragments that are too dark, animate with time.
//----------------------------------------------------------------

void main()
{
	vec4 texture = texture(texUnit, DataIn.texCoord);
 
    // TODO Don't draw this pixel if it is "dissolved"
	//    HINT: use 'texture.r'
	//    HINT: use 'time'
	//    HINT: use 'discard'
	if(texture.r < abs(sin(time)))
		discard;

	// Set the specular term to black
	vec4 spec = vec4(0.f);

	// Normalize both input vectors
	vec3 n = normalize(DataIn.normal);
	vec3 e = normalize(vec3(DataIn.eye));

	float intensity = max(dot(n, l_dir), 0.f);

	// If the vertex is lit compute the specular color
	if(intensity > 0.f)
	{
		// Compute the half vector
		vec3 h = normalize(l_dir + e);	
		// Compute the specular term into spec
		float intSpec = max(dot(h, n), 0.f);
		spec = specular * pow(intSpec, shininess);
	}

	//compute basic color
	vec4 tint = ambient + intensity * diffuse;

	//combine tint, textureColor and Specular
	vec4 fullColor = texture*tint + spec;

	//output corrected color
	outputF = clamp(fullColor, 0.0f, 1.0f);
}