#version 330

layout (std140) uniform Matrices
{
	mat4 m_pvm;
	mat4 m_viewModel;
	mat3 m_normal;
};

layout (std140) uniform Lights
{
	vec3 l_dir;		// camera space
};

in vec4 position;	// local space
in vec3 normal;		// local space
in vec3 tangent;	// local space
in vec3 bitangent;	// local space
in vec2 texCoord;

out Data
{
	vec3 normal;
	vec3 eye;
	vec3 light;
	vec2 texCoord;
} DataOut;

//----------------------------------------------------------------
// TODO:
// 1) Transform input normal, tangent and bitangent vectors with normal matrix.
// 2) Transform light into tangent space.
// 3) Transform eye into tangent space.
// 4) Don't forget to send computed data to fragment shader
// 5) Optional - compute bitangent vector from normal and tangent instead of using the passed one.
//----------------------------------------------------------------

void main()
{
	// Transform local coordinate space into camera space using the normal matrix
	
	//TODO 1) Transform input normal, tangent and bitangent vectors with normal matrix
	vec3 n = normalize(m_normal * normal);
	vec3 t = normalize(m_normal * tangent);

#if 1 // direct usage of provided bitangent
	vec3 b = normalize(m_normal * bitangent);

#else // OR by computing the last vector to be orthogonal to the first two - result will be the same

	//TODO 5) Compute bitangent vector from normal and tangent instead of using the passed one.
	//   Hint: use 'cross' function
	vec3 b = normalize(cross(n, t));

#endif


	DataOut.normal = vec3(0.f, 0.f, 1.f); // Not needed, send for reference

	// Transform light and eye vectors from camera space into tangent space using this matrix:
	// tx ty tz
	// bx by bz
	// nx ny nz

	//TODO 2) Transform light vector from camera space to tangent space
	//   Hint: use 'dot'
	vec3 lightTangent;
	lightTangent.x = dot(l_dir, t);
	lightTangent.y = dot(l_dir, b);
	lightTangent.z = dot(l_dir, n);
	
	
	// Transform eye into tangent space...
	//    Compute vector 'to-camera' in the camera space
	vec4 vertexView = -(m_viewModel * position);
	
	// Convert homogeneous position into euclidean
	vec3 vertexViewEuklid = vertexView.xyz / vertexView.w;
	
	//TODO 3) Transform this vector into tangent space
	//   Hint: use 'dot'
	vec3 eye;
	eye.x = dot(vertexViewEuklid, t);
	eye.y = dot(vertexViewEuklid, b);
	eye.z = dot(vertexViewEuklid, n);
	

	DataOut.texCoord = texCoord;
	
	//TODO 4) send computed vectors to fragment shader
	DataOut.light = lightTangent;
	DataOut.eye = eye;

	gl_Position = m_pvm * position;	
}